<?php 

class CF7DUDropbox {

	private $api_token = "";

	private $submission_folder = "";

	private $submission_id = "";

	private $file_id = "";

	private $expiration = "";

	private $upload_session_id = "";

	function __construct($file_id = "", $submission_id = "", $upload_session = "") {
		if ($file_id == "") return false;

		$this->file_id = $file_id;

		if($submission_id == "") {
			// Generate Submission ID
			$submission_id = rand(99999, 9999999);
		}

		if($upload_session != "") {
			$this->upload_session_id = $upload_session;
		}

		$this->submission_id = $submission_id;

		$cf7du_files = get_option( 'cf7du_files' );

		if(isset($cf7du_files['cf7du_dropbox_token']) && $cf7du_files['cf7du_dropbox_token'] != '') {
			$this->api_token = $cf7du_files['cf7du_dropbox_token'];
		}

		if(isset($cf7du_files['cf7du_dropbox_link_expires']) && $cf7du_files['cf7du_dropbox_link_expires'] != '') {
			$this->expiration = $cf7du_files['cf7du_dropbox_link_expires'];
		}
	}

	function create_folder() {
		$folder_name = "Submission " . $this->submission_id;

		$this->submission_folder = $folder_name;

		$this->api_create_folder($folder_name);
	}

	private function api_create_folder($folder_name) {

		$api_url = "https://api.dropboxapi.com/2/files/create_folder_v2";

		$headers = array( 
	        "Authorization: Bearer " . $this->api_token, 
	        "Content-Type: application/json"
	    );

	    $post = array(
	    	'path' => "/{$folder_name}",
	    	'autorename' => false
	    );

	    $ch = curl_init(); 
	    curl_setopt($ch, CURLOPT_URL, $api_url); 
	    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
	    curl_setopt($ch, CURLOPT_POST, true);
	    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post));
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

	    $data = curl_exec($ch); 

	}

	function upload_file($filepath) {

		return $this->api_upload_file($filepath);

	}

	private function api_upload_file($filepath = '') {

		if($this->upload_session_id == '') {
			// Start new uploading session

			$api_url = "https://content.dropboxapi.com/2/files/upload_session/start";

			$args = array(
				'close' => false
			);

			$headers = array( 
		        "Authorization: Bearer " . $this->api_token, 
		        "Dropbox-API-Arg: " . json_encode($args),
		        "Content-Type: application/octet-stream"
		    );

		    $fp = fopen($filepath, 'rb');
			$size = filesize($filepath);

		    $ch = curl_init(); 
		    curl_setopt($ch, CURLOPT_URL, $api_url); 
		    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		    curl_setopt($ch, CURLOPT_PUT, true);
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
			curl_setopt($ch, CURLOPT_INFILE, $fp);
			curl_setopt($ch, CURLOPT_INFILESIZE, $size);
		    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		    $data = curl_exec($ch); 

		    $data_decode = json_decode($data);

		    if(isset($data_decode->session_id)) {
		    	$this->upload_session_id = $data_decode->session_id;

		    	return $data_decode->session_id;
		    } else {
		    	return false;
		    }

		} else {
			// Append data to session

			$api_url = "https://content.dropboxapi.com/2/files/upload_session/append_v2";

			$chunk = $_POST['chunk'];

			$args = array(
				'cursor' => array(
					'session_id' => $this->upload_session_id,
					'offset' => $chunk * 1000000
				),
				'close' => false
			);

			$headers = array( 
		        "Authorization: Bearer " . $this->api_token, 
		        "Dropbox-API-Arg: " . json_encode($args),
		        "Content-Type: application/octet-stream"
		    );

			$fp = fopen($filepath, 'rb');
			$size = filesize($filepath);

		    $ch = curl_init(); 
		    curl_setopt($ch, CURLOPT_URL, $api_url); 
		    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		    curl_setopt($ch, CURLOPT_PUT, true);
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
			curl_setopt($ch, CURLOPT_INFILE, $fp);
			curl_setopt($ch, CURLOPT_INFILESIZE, $size);
		    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		    $data = curl_exec($ch); 

		    return $this->upload_session_id;
		}
	}

	function finish_session($session_id, $filename, $size) {
		$this->api_finish_session($session_id, $filename, $size);
	}

	private function api_finish_session($session_id, $filename, $size) {
		$api_url = "https://content.dropboxapi.com/2/files/upload_session/finish";

		$args = array(
			'cursor' => array(
				'session_id' => $session_id,
				'offset' => (int)$size
			),
			'commit' => array(
				'path' => '/' . $filename
			)
		);

		$headers = array( 
	        "Authorization: Bearer " . $this->api_token, 
	        "Dropbox-API-Arg: " . json_encode($args),
	        "Content-Type: application/octet-stream"
	    );

	    $ch = curl_init(); 
	    curl_setopt($ch, CURLOPT_URL, $api_url); 
	    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

	    $data = curl_exec($ch); 

	    $data_decode = json_decode($data);
	}

	function get_share_link($filepath) {

		$current_date = date('Y-m-d H:i:s');

		if($this->expiration != "" && $this->expiration != "never") {
			switch ($this->expiration) {
				case 'hour':
					$expiration = date('o-m-d\TH:i:s\Z', strtotime($current_date . ' +1 hour'));
					break;
				
				case 'day':
					$expiration = date('o-m-d\TH:i:s\Z', strtotime($current_date . ' +1 day'));
					break;
				
				case 'week':
					$expiration = date('o-m-d\TH:i:s\Z', strtotime($current_date . ' +7 days'));
					break;
				
				case 'month':
					$expiration = date('o-m-d\TH:i:s\Z', strtotime($current_date . ' +1 month'));
					break;

				default:
					$expiration = "";
					break;
			}
		} else {
			$expiration = "";
		}

		return $this->api_get_share_link($filepath, $expiration);
	}

	private function api_get_share_link($filepath, $expiration = "") {
		$api_url = "https://api.dropboxapi.com/2/sharing/create_shared_link_with_settings";

		$link_settings = array();

		$headers = array( 
	        "Authorization: Bearer " . $this->api_token, 
	        "Content-Type: application/json"
	    );

		$link_settings['requested_visibility'] = "public";

		if($expiration != "") {
			$link_settings['expires'] = $expiration;			
		}

	    $post = array(
			'path' => '/' . $filepath,
			'settings' => $link_settings
	    );

	    $ch = curl_init(); 
	    curl_setopt($ch, CURLOPT_URL, $api_url); 
	    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
	    curl_setopt($ch, CURLOPT_POST, true);
	    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post));
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

	    $data = curl_exec($ch); 

	    $data_decode = json_decode($data);

	    if(isset($data_decode->url)) {
	    	return $data_decode->url;
	    } else {
	    	return false;
	    }
	}
}